#include "MyTools.h"

#include <stdint.h>
#include <time.h>
#include <chrono>
#include <fstream>
#include <iostream>
#include <string>
#include <time.h>
#include <stdio.h>

#include <numeric>
#include "Timer.h"

namespace MyTools
{

    std::ofstream logOut;

    void FileLoggerSingleton::OpenLogFile(const std::string &FN)
    {
        logOut.open(FN, std::ios_base::out);
    }

    void FileLoggerSingleton::CloseLogFile()
    {
        if (logOut.is_open())
        {
            logOut.close();
        }
    }

    std::string GetCurDateTime()
    {
        auto cur = std::chrono::system_clock::now();
        time_t time = std::chrono::system_clock::to_time_t(cur);
        char *buf = ctime(&time);
        return std::string(buf);
    }

    void FileLoggerSingleton::WriteToLog(const std::string &str)
    {
        if (logOut.is_open())
        {
            logOut << GetCurDateTime() << " - " << str << std::endl;
        }
    }

    void FileLoggerSingleton::WriteToLog(const std::string &str, int n)
    {
        if (logOut.is_open())
        {
            logOut << GetCurDateTime() << " - " << str << n << std::endl;
        }
    }

    void FileLoggerSingleton::WriteToLog(const std::string &str, double d)
    {
        if (logOut.is_open())
        {
            logOut << GetCurDateTime() << " - " << str << d << std::endl;
        }
    }

    void LoggerSingleton::OpenLogFile(const std::string &FN)
    {
        Timer::getInstance().start();
        LoadLoggerSingletone().OpenLogFile(FN);
        times.push_back(Timer::getInstance().end());
    }

    void LoggerSingleton::CloseLogFile()
    {
        Timer::getInstance().start();
        LoadLoggerSingletone().CloseLogFile();
        times.push_back(Timer::getInstance().end());
    }

    void LoggerSingleton::WriteToLog(const std::string &str)
    {
        Timer::getInstance().start();
        LoadLoggerSingletone().WriteToLog(str);
        times.push_back(Timer::getInstance().end());
    }

    void LoggerSingleton::WriteToLog(const std::string &str, int n)
    {
        Timer::getInstance().start();
        LoadLoggerSingletone().WriteToLog(str, n);
        times.push_back(Timer::getInstance().end());
    }

    void LoggerSingleton::WriteToLog(const std::string &str, double d)
    {
        Timer::getInstance().start();
        LoadLoggerSingletone().WriteToLog(str, d);
        times.push_back(Timer::getInstance().end());
    }

    LoggerSingleton::~LoggerSingleton()
    {
        if (times.size())
        {
            std::cout << "Среднее время выполнения операций: " << std::accumulate(times.begin(), times.end(), 0.0) / times.size() << " секунд" << std::endl;
        }
    }

    FileLoggerSingleton& LoggerSingleton::LoadLoggerSingletone()
    {
        if (!_logger)
        {
            _logger = &FileLoggerSingleton::getInstance();
        }

        return *_logger;
    }

} // namespace MyTools
